/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * crc32.c
 *
 * crc32 API.
 */

#include <main.h>

/*
 * Initialize the crc32 table.
 */
void crc32_init(u_int32_t *table, u_int32_t *crc32sum)
{
    u_int32_t crc;
    const u_int32_t p = 0xEDB88320L;
    int	i, j;

    i = 0;
    while( i < 256 )
    {
        crc = i;
	j = 8;
	while( j > 0 )
        {
            if (crc & 1)  crc = (crc >> 1) ^ p;
            else          crc >>= 1;
	    j--;
        }
        table[i] = crc;
	i++;
    }
    
    *crc32sum = 0xFFFFFFFF;
}


/*
 * Process 'n' bytes from the 'bytes' array.
 */
void crc32_process(u_int32_t *table, u_int32_t *crc32sum, unsigned char *bytes, u_int32_t no)
{
    register uint i;

    i = 0;
    while( i < no )
    {
        *crc32sum =
	    (((*crc32sum) >> 8) & 0x00FFFFFF) ^ table[ ((*crc32sum)^bytes[i]) & 0xFF ];
	i++;
    }
}


/*
 * Finalize the crc32 checksum.
 */
void crc32_finalize(u_int32_t *table, u_int32_t *crc32sum)
{
    *crc32sum ^= 0xFFFFFFFF;
}
