/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */


#ifndef _RDA_MAIN_H
#define _RDA_MAIN_H

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/vfs.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <signal.h>
#include <getopt.h>
#include <stdarg.h>
#ifndef __USE_LARGEFILE64
#	define __USE_LARGEFILE64
#endif
#include <fcntl.h>
#include <linux/unistd.h>
//#include <syscall.h>

#include "md5.h"

#define PROGNAME		"rda"
#define VERSION			"0.2.1c"

#define DEF_PORT		6699
#define DEF_BLKSZ		1024
#define DEF_ERR_BLKSZ		512	/* This is the minimum block to be read from */
					/* a block device by kernel. */
#define ERR_RETRIES		1
#define DEF_BUFSZ		DEF_BLKSZ

    /* MAX_BLKSZ should be always less than MIN_SPANSZ or else span fails */
#define MAX_BLKSZ		83886080	/* 80M */
#define MIN_SPANSZ		104857600	/* 100M */

#define DEF_LOG			"./rda.log"

#define SOCK_WR_RETRIES		20

#define LOOPS_REPORT		100

#define SPAN_FILE_DIGITS	3	/* file suffix number length for span. */

#ifdef COLOR
#	define BLD			"\033[1m"
#	define RED			"\033[1;31m"
#	define BLU			"\033[1;34m"
#	define GRN			"\033[1;32m"
#	define CYN			"\033[1;36m"
#	define RST			"\033[0m"
#else
#	define BLD
#	define RED
#	define BLU
#	define GRN
#	define CYN
#	define RST
#endif

#define VERSION_HEADER	\
    PROGNAME " version " VERSION 							\
    "\nBy Chris Boubalos and Stefanos Koutsoutos."					\
    "\n\nCopyright (C) 2002 MD5 S.A.\n"							\
    "This program is free software; you may redistribute it under the terms of\n"	\
    "the GNU General Public License.  This program has absolutely no warranty.\n\n"

#define BUF_INITONCE(b, sz)		{ if(!b) b = malloc(sz); }

#include "report.h"
#include "client.h"
#include "server.h"

struct _state {

    unsigned char type;
#define TP_SRV	1
#define TP_CLI	2
//#define TP_LCL	4		// TODO

    int reportlevel;

    unsigned char *buf;
    uint buflen;
    
    union {
	ServerState s;
	ClientState c;
    } u;
    
} State;

#define Server( member )	(State.u.s.member)
#define Client( member )	(State.u.c.member)
//#define Local( member )		(State.u.l.member)

#define Global( member )	( (State.type == TP_CLI) ? Client(member) : Server(member) )


#define llseek _llseek

#include "api.h"

#endif /* main.h */
