/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * init.c
 *
 * Network and file open/close routines, signal handlers and the like.
 */

#include <main.h>


/*
 * Open the network connection specified in 'State'.
 */
int initnet(void)
{
    
    if( State.type == TP_CLI )
    {
	Client(addr).sin_family = PF_INET;
	Client(addr).sin_addr.s_addr = inet_addr(Client(ip));
	Client(addr).sin_port = htons(Client(port));
    }
    else
    {
	Server(addr).sin_family = PF_INET;
	Server(addr).sin_addr.s_addr = INADDR_ANY;
	Server(addr).sin_port = htons(Server(port));
    }

    report(2,"Initializing network connection.\n");
    
    if( (Global(sock) = socket(AF_INET, SOCK_STREAM, 0)) < 0 ){
	report(0,RED"Failed "RST"opening a socket: %s.\n", strerror(errno));
	return -1;
    }
    
    if( State.type == TP_CLI )
    {
	if( connect(Client(sock), (struct sockaddr *)&Client(addr),
				    sizeof(Client(addr))) < 0 )
	{
	    report(0,RED"Failed"RST" connecting: %s.\n", strerror(errno));
	    report(0,"Is the server running??\n");
	    report(1,"Remote host: %s @ %u.\n", Client(ip), Client(port));
	    return -1;
	}
    }
    else
    {
        if( bind(Server(sock), (struct sockaddr *)&Server(addr),
				    sizeof(Server(addr))) < 0 )
	{
	    report(0,RED"Failed"RST" binding: %s.\n", strerror(errno));
	    return -1;
	}
    
	if( listen(Server(sock),1) < 0 )
	{
	    report(0,RED"Failed"RST" listening on master socket: %s\n", strerror(errno));
	    return -1;
	}
    }
    
    return 0;
}



/*
 * Server mode: wait for client to connect. If a client is connected close his connection.
 */
int waitclient(void)
{
    int len;
    struct sockaddr_in addr;
    char *mac = NULL;
    
    if( Server(c_sock) > 0 ){
	report(2,"Closing already connected client's connection.\n");
	close( Server(c_sock) );
        Server(c_sock) = -1;
    }
    
    report(2,"Waiting a client to connect.\n");
restart:
    Server(c_sock) = accept(Server(sock), (struct sockaddr *)&addr, &len);
    if( Server(c_sock) < 0 ){
	report(0,RED"Error"RST" while waiting to accept connections: %s\n", strerror(errno));
	return -1;
    }

    if( !Server(mac_set) ){
	mac = macof(addr.sin_addr);
	if(mac) strcpy(Server(mac), mac);
	Server(mac_set) = 1;
    }
    
    report(2,"Mac address of the client is: %s\n", Server(mac));
    
    if( Server(keithe).sin_addr.s_addr )
    {
	// since we're here, we've set this address before...
	// If the IPs don't match just reject the !(@)$^%@)&#$)@* and restart the function.
	if( Server(keithe).sin_addr.s_addr != addr.sin_addr.s_addr ){
	    close(Server(c_sock));
	    report(0,RED"Warning:"RST" Attempt to connect "BLD"NOT"RST" by my original client...\nReject and restart...\n");
	    goto restart;
	}
    }
    
    getsockname(Server(c_sock), (struct sockaddr *)&Server(dwthe),  &len);
    Server(keithe) = addr;
    
    report(1,"Client connected from: %s @ %u\n", inet_ntoa(addr.sin_addr), ntohs(addr.sin_port) );
    return 0;
}




/*
 * close the network connection.
 */
int closenet(void)
{
    
    report(2,"Closing network connection.\n");
    
    if( State.type == TP_CLI )
    {
        if( Client(sock) >= 0 ) close( Client(sock) );
        Client(sock) = -1;
        bzero(&Client(addr), sizeof(Client(addr)));
    } else
    {
        if( Server(sock) >= 0 ) close( Server(sock) );
        Server(sock) = -1;
        if( Server(c_sock) >= 0 ) close( Server(c_sock) );
        Server(c_sock) = -1;
        bzero(&Server(addr), sizeof(Server(addr)));
    }

    return 0;
}


/*
 * close the input descriptor.
 */
int closeinput(void)
{
    report(2,"Closing input file.\n");
    if( Client(infd) >= 0 ) close( Client(infd) );
    Client(infd) = -1;
    return 0;
}



/*
 * close the input descriptor.
 */
int closeoutput(void)
{
    report(2,"Closing output file.\n");
    if( Server(outfd) >= 0 ) close( Server(outfd) );
    Server(outfd) = -1;
    return 0;
}



/*
 * Cleanup program's resources. This is used in non-normal program
 * shutdwon mainly.
 */
int cleanup(void)
{
    closenet();

    if( State.type == TP_CLI )	/// client.
    {
	closeinput();
	
	if( Client(transf) )
	{
	    report(1,"Re-inializing network for error sending.\n");
	    initnet();
    
	    report(0,"Send error message to the server.\n");

	    Client(report)->error = 1;
	    Client(report)->_errno = errno;
	    
		/* Don't check for errors; what can we do, anyway? */
	    write(Client(sock), Client(report), sizeof(Report)+Client(report)->cmd_len-1 );
	    
	    report(1,"Error description and rest info sent.\n");
	}
    } else	/// server.
	closeoutput();

    printf("\033[?25h");	// restore the user's cursor!
    return 0;
}



/*
 * We use only one signal handler for any signal handled.
 */
void sig_handler(int signo)
{
    if( State.type == TP_CLI ) Client(report)->signaled = 1;
    switch(signo)
    {
	case SIGSEGV:
	    report(1,BLU"Segmentation Violation"RST" Signal caught --> Safe exit.\n");
	    cleanup();
	    exit(SIGSEGV);
	case SIGINT:
	    report(1,BLU"Interrupted"RST"! --> Safe exit.\n");
	    cleanup();
	    exit(SIGINT);
	case SIGPIPE:
	    report(2,BLU"Pipe I/O signal"RST" caught --> Ignore.\n");
	default:
	    return;
    }
}
