/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * log.c
 *
 * Logging functions and screen report (server only).
 */

#include <main.h>


/*
 * Server mode: check the remotely sent md5 checksum and the localy produced.
 */
int checkmd5(void)
{
    register int i = 0;

    while( i < 16 ){
	if( Server(md5sum)[i] != Server(md5sum_remote)[i] ){
	    return -1;
	}
	i++;
    }
    return 0;
}




/*
 * Server mode: print the md5 checksum supplied.
 */
void hex(unsigned char *sum, unsigned int no, FILE *out)
{
    char bin2hex[] = { '0', '1', '2', '3', '4',
		       '5', '6', '7', '8', '9',
		       'a', 'b', 'c', 'd', 'e',
		       'f' };
    register int i = 0;
    
    while( i < 16 ){
	putc( bin2hex[ ((sum[i] & 0xf0 ) / 16) % 16 ], out );
	putc( bin2hex[ ( sum[i] & 0x0f ) % 16 ], out );
	i++;
    }
}


/*
 * Save a log file when done.
 */
void savelog(time_t st, time_t nd, int argc, char **argv, u_int32_t dur,
	     float rate)
{
    FILE *fp;
    register int i = 0;
    struct utsname uts;
    char *c1, *c2;
    
    if( !Server(outputf) ) return;
    
    if( !(fp = fopen(Server(logf),"w")) ){
	report(1,BLU"Failed"RST" opening log file: %s :: %s\n",
	    Server(logf), strerror(errno));
	return;
    }

    fprintf(fp, PROGNAME " " VERSION " report --

Start:       %sEnd:         %s
Server Cmd:         ", (c1 = strdup(ctime(&st))), (c2 = strdup(ctime(&nd))));

    if(c1) free(c1);
    if(c2) free(c2);

    i = 0;
    while( i < argc )
    {
	fprintf(fp, "%s ", argv[i]);
	i++;
    }
    fprintf(fp,"\nClient Cmd:         %s\n", Server(report)->cmd );
    fprintf(fp,"Read attributes: blocksize = %u,  count = ",
		Server(report)->blocksize );

    if( !Server(report)->count ) fputs("N/A\n",fp);
    else fprintf(fp,"%u\n", Server(report)->count );
    
    fputs("\n Checksums:                         MD5                      CRC32"
	    "\nServer:               ", fp);
    if( Server(md5) ) hex(Server(md5sum),16,fp);
    else fputs("              N/A               ",fp);
    if( Server(crc32) ) fprintf(fp,"    %lu\n", (unsigned long)Server(crc32sum));
    else fputs("        N/A\n",fp);
    
    fputs("Client:               ", fp);
    if( Server(report)->md5 ) hex(Server(md5sum_remote),16,fp);
    else fputs("              N/A               ",fp);
    if( Server(report)->crc32 ) fprintf(fp,"    %lu\n", (unsigned long)Server(crc32sum_remote));
    else fputs("        N/A\n",fp);
    
    fputs("Match:                ", fp);
    if( Server(md5) && Server(report)->md5 ){
	if( checkmd5() < 0 ){
	    fputs("       Checksum don't match      ",fp);
	} else {
	    fputs("           They match!           ",fp);
	}
    } else fputs("              N/A                ", fp);
        
    if( Server(crc32) && Server(report)->crc32 ){
	if( checkcrc32() < 0 ){
	    fputs("   Checksum don't match\n\n",fp);
	} else {
	    fputs("   They match!\n\n",fp);
	}
    } else fputs("       N/A\n\n", fp);
        
    fprintf(fp,"Bytes received from network:         %llu
Bytes localy written:                %llu
Bytes reported to be sent by client: %llu

 %lu seconds -- %.2f Kbytes / sec\n\n",
	Server(tot_in), Server(tot_out), Server(remote_in), (unsigned long)dur, rate);

    if( uname(&uts) == 0 )
	fprintf(fp, "System:  %s (%s)\nHost:    %s (%s)\n", uts.sysname,
	    uts.release, uts.nodename, uts.machine);

    fprintf(fp, "Local ip:    %s\nRemote ip:   %s   (MAC: %s)\n\n",
	strdup( inet_ntoa(Server(dwthe).sin_addr) ),
	strdup( inet_ntoa(Server(keithe).sin_addr) ),
	(Server(mac)[0])?Server(mac):"N/A" );

    if( !Server(report)->badblocks_no ) fputs("No badblocks were found.\n\n", fp);
    else
    {
	register unsigned long t = 0;
	int err = 0;
	fprintf(fp,"Badblocks found:\n");
	i = 0;
	while( i < Server(report)->badblocks_no )
	{
	    if( Server(bbdefs)[i].start ) fprintf(fp, "\t-> @ %llu", Server(bbdefs)[i].bb_offt); 
	    else if( Server(bbdefs)[i].end ){
		fprintf(fp, " to  %llu\n", Server(bbdefs)[i].bb_offt);
		if(!i) err = 1;
		else t += (unsigned long)(Server(bbdefs)[i].bb_offt - Server(bbdefs)[i-1].bb_offt);
	    }
	    else {
		fprintf(fp, "\t-> @ %llu\n", Server(bbdefs)[i].bb_offt);
		t++;
	    }
	    i++;
	}
	fprintf(fp,"Total badblocks: %lu\n", t );
	if(err) fputs("There were some errors in the list of badblocks.\n",fp);
    }

    fclose(fp);
}
