/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * loop_c.c
 *
 * Functions invoked in the main program's loop context (client side).
 */

#include <main.h>


/*
 * Read the input from the opened file.
 */
int readinput_c(void)
{
    int len = 0;
    
    State.buflen = 0;
    
    if( Client(count) != 0 ) if( Client(tot_in) >= (Client(count)*Client(blocksize)) ){
	report(2,"Read specified count (%d) of blocks from input.\n", Client(count) );
	return 0;
    }

    BUF_INITONCE( State.buf, Client(blocksize) );
    if( !State.buf ){
	report(0,RED"Failed"RST" initailizing the buffer: %s.\n", strerror(errno));
	return -1;
    }
    
    report(3,"Reading from the input.\n");
    
    len = read(Client(infd), State.buf, Client(blocksize));
    if( len < 0 ){
	report(0,"Error reading input: %s.\n", strerror(errno));
	return (Client(skip_errors)) ? skip_error() : -1;
    } else if( len == 0 ){
	report(2,BLU"EOF"RST" reached in input.\n");
	return 0;
    }
    
    State.buflen = len;
    report(3,"Read another: %d bytes (Blocksize: %d).\n", len, Client(blocksize));

    Client(tot_in) += len;
    
    return len;
}



/*
 * Send the input buffer to the remote host.
 */
int sendinput(void)
{
    int len = 0, tlen = 0, retry = 0;
    
    report(3,"Sending data to remote host.\n");
    
    if( State.buflen < 1 ){
	report(2,"Nothing left to be sent.\n");
	return 0;
    }
    
    for(;;)
    {
	len = write( Client(sock), State.buf+tlen, State.buflen-tlen );
        if( len < 0 ){
    	    report(0,RED"Error"RST" writting to socket: %s.\n", strerror(errno));
    	    return -1;
        } else if(len == 0){
	    retry++;
	    report(0,"Unknown exception while writting --> ");
	    
	    if( retry > SOCK_WR_RETRIES ){
		report(0,"Expired max (%d). Abort write.\n", SOCK_WR_RETRIES);
	    } else
		report(0,"Retry.\n");
	}
    
        Client(tot_out) += len;	/* do this here, not after the loop whith 'tlen'
				 * since we can loose some data if an exception
				 * occures later. */
	report(3,"Data sent: %d bytes of %d.\n", len, State.buflen);
	tlen += len;
	if( tlen >= State.buflen ) break;
    }
    
    return tlen;
}



/*
 * Send the total bytes count and the md5sum to
 * the remote host.
 *
 * IMPORTANT: We have to send all bytes and md5sum at one packet,
 * since the server should take it at once.
 */
int sendrest(void)
{
    int len = 0;
    
    report(2,"Sending report.\n");
    
    report(2,"Copying to bufffer.\n");
    
    bcopy( Client(md5sum), Client(report)->md5sum, 16 );
    bcopy( &Client(crc32sum), Client(report)->crc32sum, 4 );
    
    Client(report)->md5 = Client(md5);
    Client(report)->crc32 = Client(crc32);
    
    Client(report)->blocksize = Client(blocksize);
    Client(report)->count = Client(count);
    Client(report)->tot = Client(tot_in);
    
    report(2,"Sending buffer.\n");
    
    len = write( Client(sock), Client(report), sizeof(Report)+Client(report)->cmd_len-1 );
    if( len < 0 ){
        report(0,"Error writting to socket: %s.\n", strerror(errno));
        return -1;
    } else if( len < (sizeof(Report)+Client(report)->cmd_len-1) ){
	report(0,"Operation (sending buffer) "BLU"only partially complete"RST" while sending report.\n");
    }
    
    report(2,"Sending BadBlocks' list.\n");
    
    if( !Client(report)->badblocks_no )
    {
	report(2,"No badblocks list to sent.\n");
	return len;
    }
    
    len = write( Client(sock), Client(bbdefs), sizeof(BBdef)*Client(report)->badblocks_no );
    if( len < 0 ){
        report(0,RED"Error"RST" writting to socket: %s.\n", strerror(errno));
        return -1;
    } else if( len < (sizeof(BBdef)*Client(report)->badblocks_no) ){
	report(0,"Operation (sending buffer) "BLU"only partially complete"RST" while sending badblocks' list.\n");
    }
    
    report(2,"Report and list sent.\n");
    return len;
}
