/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * loop_s.c
 *
 * Functions invoked in the main program's loop context (server side).
 */

#include <main.h>


/*
 * Wait on a socket for read.
 */
static inline int waitsockread(int sock)
{
    int sr;
    fd_set fdset;
    struct timeval tm;

    for(;;)
    {
        FD_ZERO(&fdset);
	FD_SET(Server(c_sock),&fdset);

	tm.tv_sec = 1;
	tm.tv_usec = 0;

	sr = select(Server(c_sock)+1, &fdset, 0, 0, &tm);
	if( sr < 0 ){
	    report(0,"An "RED"error"RST" occured while waiting to read: %s.\n", strerror(errno));
	    return -1;
	}
	if( sr > 0 )
	    if( FD_ISSET(Server(c_sock), &fdset) )
		return 0;	/* we've got data!!! */
    }
    report(0,"Unreachable code: %s @ %d\n", __FILE__, __LINE__);
    return -1;
}



/*
 * Read the input from the connected client.
 */
int readinput_s(void)
{
    int len = 0;
    
    State.buflen = 0;
    
    BUF_INITONCE( State.buf, DEF_BUFSZ );
    if( !State.buf ){
	report(0,RED"Failed"RST" initializing the buffer: %s.\n", strerror(errno));
	return -1;
    }
    
    report(3,"Reading from the net.\n");

    if( waitsockread(Server(c_sock)) < 0 ) return -1;

    len = read(Server(c_sock), State.buf, DEF_BUFSZ);
    if( len <= 0 ){
	report(2,"Client closed socket.\n");
	report(3,"read() return: %d --> error:%s.\n", len, strerror(errno));
	return 0;
    }
    
    State.buflen = len;
    report(3,"Read another: %d bytes.\n", len);

    Server(tot_in) += len;
    
    return len;
}



/*
 * Write the buffer to the output file.
 */
int writeoutput(void)
{
    int len = 0, tlen = 0;
    static int spanin = 0;
    
    report(3,"Writting data to file.\n");
    
    if( State.buflen < 1 ){
	report(2,"Nothing left to be written.\n");
	return 0;
    }
    
    /* Check if span is enabled and if yes open the next file when span size is to be exceeded. */
    if( Server(spansz) )
    {
	if( (spanin + State.buflen) > Server(spansz) )
	{
	    report(2,"Creating next span file (Count: %d).\n", Server(span_cnt)+1);
	    spanin = 0;
	    close(Server(outfd));
	    Server(span_cnt)++;
	    sprintf(Server(outputf), Server(span_fmt), Server(span_base), Server(span_cnt));
	    
	    if( (Server(outfd) = open(Server(outputf), O_WRONLY | O_LARGEFILE | O_CREAT, 0666 )) < 0 ){
		report(0,RED"Failed"RST" opening file: %s :: %s.\n", Server(outputf), strerror(errno));
		return -1;
	    }
	}
    }
    
    for(;;)
    {
	len = write(Server(outfd), State.buf+tlen, State.buflen-tlen );
        if( len < 0 ){
    	    report(0,RED"Error"RST" writting to output: %s.\n", strerror(errno));
	    report(3,"FD: %d - Buf: %p - Blen: %u\n", Server(outfd), State.buf, State.buflen);
    	    return -1;
        } else if(len == 0){
	    report(0,"Unknown exception while writting --> Abort.\n");
	    return -1;
	}
    
        Server(tot_out) += len;	/* do this here, not after the loop whith 'tlen'
				 * since we can loose some data if an exception
				 * occures later. */
	report(3,"Data wrote: %d bytes of %d.\n", len, State.buflen);
	tlen += len;
	spanin += len;
	if( tlen >= State.buflen ) break;
    }
    
    return tlen;
}



/*
 * Read the total bytes count and the md5sum (not in this version) from
 * the remote client.
 */
int readrest(void)
{
    int len = 0;
    
    Server(report) = malloc(sizeof(Report));
    if(!Server(report)){
	report(0,RED"Cannot allocate memory"RST" to store the client report: %s\n", strerror(errno));
	return -1;
    }
    
    report(2,"Reading report.\n");

    if( waitsockread(Server(c_sock)) < 0 ) return -1;

    len = read(Server(c_sock), Server(report), sizeof(Report));
    if( len < 0 )
    {
	report(0,"Client closed socket.\n");
	report(3,"read() return: %d --> error:%s.\n", len, strerror(errno));
	return -1;
    }

    report(2,"Fetching command line.\n");
    
    Server(report) = realloc(Server(report), sizeof(Report)+Server(report)->cmd_len-1);
    if(!Server(report)){
	report(0,RED"Cannot allocate memory"RST" to store the client report: %s\n", strerror(errno));
	return -1;
    }
    
    len = read(Server(c_sock), ((char*)Server(report))+sizeof(Report), Server(report)->cmd_len-1);
    if( len < 0 )
    {
	report(0,"Client closed socket.\n");
	report(3,"read() return: %d --> error:%s.\n", len, strerror(errno));
	return -1;
    }
    
    if( Server(report)->error )
    {
	report(0,"\nClient exited with an error.\n");
	if( Server(report)->signaled ) report(0,"Client was signalled.\n\n");
	else if( Server(report)->badcond ) report(0,"Client encountered an unexpected condition.\n\n");
	else report(0,"The last error was: %s\n\n", strerror(Server(report)->_errno));
	
	return -1;	// so that we don't write logs and stuff....
    }
    
    
    if( !Server(report)->badblocks_no ) report(2,"No badblocks' list to fetch.\n");
    else {
	report(2,"Fetching badblocks' list.\n");
	
	Server(bbdefs) = malloc(sizeof(BBdef)*Server(report)->badblocks_no);
	if( !Server(bbdefs) ){
	    report(0,RED"Cannot allocate memory"RST" to store the badblocks's list: %s\n", strerror(errno));
	    return -1;
        }

        len = read(Server(c_sock), Server(bbdefs), sizeof(BBdef)*Server(report)->badblocks_no);
	if( len < 0 )
        {
	    report(0,"Client closed socket.\n");
	    report(3,"read() return: %d --> error:%s.\n", len, strerror(errno));
	    return -1;
        }
    }
    
    /* Copy some values we need to... */
    if( Server(report)->md5 )   bcopy( Server(report)->md5sum, Server(md5sum_remote), 16 );
    if( Server(report)->crc32 ) bcopy( Server(report)->crc32sum, &Server(crc32sum_remote), 4 );
    
    Server(remote_in) = Server(report)->tot;
    
    report(2,"All report fetched by client.\n");
    
    return len;
}
