/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * main.c
 *
 * main(), conf and misc. functions.
 */

#include <main.h>

struct option options[] = {

    { "send", no_argument, NULL, '1' },
    { "recv", no_argument, NULL, '2' },
    { "help", no_argument, NULL, 'h' },
    { "version", no_argument, NULL, 'v' },
    { "in", required_argument, NULL, 'i' },
    { "out", required_argument, NULL, 'o' },
    { "host", required_argument, NULL, 'r' },
    { "port", required_argument, NULL, 'p' },
    { "bs", required_argument, NULL, 'b' },
    { "count", required_argument, NULL, 'c' },
    { "debug", required_argument, NULL, 'd' },
    { "span", required_argument, NULL, 'a' },
    { "log", required_argument, NULL, 'l' },
    { "silent", no_argument, NULL, 's' },
    { "nolog", no_argument, NULL, 'n' },
    { "skip-errors", no_argument, NULL, 'e' },
    { "checksum", required_argument, NULL, 'm' }
};



/*
 * Get a number of bytes. You may specify k,m,g for kilo-, mega-, giga- bytes
 * respectively. The function will set 'err' to 1, if any error occured.
 */
static inline u_int32_t get_no32(char *str, int *err)
{
    u_int32_t ret = 0;
    char *end;
    errno = 0;

    ret = strtol( str, &end, 10 );
    if( errno == ERANGE ){
	*err = 1;
	report(0,"Number out of range.\n");
	return 0;
    }
    
    switch( *end )
    {
	case 'g':
	case 'G':
	    ret *= 1024;
	case 'm':
	case 'M':
	    ret *= 1024;
	case 'k':
	case 'K':
	    ret *= 1024;
    }

    return ret;
}


/*
 * parse the command line options.
 */
int parsecmd(int argc, char **argv)
{
    int c, len, algos = 0, err;
    char *cmdline = NULL;
    
    if(argc==1){
	printhelp();
	exit(0);
    }
    
    while( (c = getopt_long(argc, argv, "hvi:r:p:b:d:c:o:a:l:snet:m:", options, NULL)) != -1 )
	switch(c)
	{
	    case '1':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		break;
	    case '2':
		if( State.type == TP_CLI ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_SRV;
		break;
	    case 'i':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		Client(inputf) = strdup(optarg);
		break;
	    case 'o':
		if( State.type == TP_CLI ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_SRV;
		Server(outputf) = strdup(optarg);
		break;
	    case 'r':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		Client(ip) = strdup(optarg);
		break;
	    case 'p':
		Global(port) = (unsigned short)atoi(optarg);
		break;
	    case 'b':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		err = 0;
		Client(blocksize) = get_no32(optarg, &err);
		if(err){
		    report(0,"Bad blocksize specification.\nAbort.\n\n");
		    return -1;
		}
		break;
	    case 'c':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		err = 0;
		Client(count) = get_no32(optarg, &err);
		if(err){
		    report(0,"Bad count specification.\nAbort.\n\n");
		    return -1;
		}
		break;
	    case 'e':
		if( State.type == TP_SRV ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_CLI;
		Client(skip_errors) = 1;
		break;
	    case 'd':
		State.reportlevel = atoi(optarg);
		break;
	    case 'a':
		if( State.type == TP_CLI ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_SRV;
		err = 0;
		Server(spansz) = get_no32(optarg, &err);
		if(err){
		    report(0,"Bad spansize specification.\nAbort.\n\n");
		    return -1;
		}
		break;
	    case 'l':
		if( State.type == TP_CLI ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_SRV;
		Server(logf) = strdup(optarg);
		break;
	    case 'n':
		if( State.type == TP_CLI ){
		    report(0,"Mixed server and client arguments.\nAbort.\n\n");
		    return -1;
		}
		State.type = TP_SRV;
		if( Server(logf) ) free( Server(logf) );
		Server(logf) = NULL;
		break;
	    case 'm':
		algos = 1;
		if( !strcmp(optarg,"both") )
		    Global(md5) = Global(crc32) = 1;
		else if( !strcmp(optarg,"md5") ){
		    Global(md5) = 1;
		    Global(crc32) = 0;
		} else if( !strcmp(optarg,"crc32") ){
		    Global(md5) = 0;
		    Global(crc32) = 1;
		}
		else if( !strcmp(optarg,"none") )
		    Global(md5) = Global(crc32) = 0;
		else {
		    report(0,"Bad checksum algorythm specified.\nAbort..\n\n");
		    return -1;
		}
		break;
	    case 's':
		State.reportlevel = -1;
		break;
	    case 'v':
		report(0,VERSION_HEADER);
		exit(0);
		break;
	    case 'h':
	    default:
		printhelp();
		exit(0);
	}
	
    if(!algos) Global(md5) = 1;
    
    if( State.type == TP_CLI )
    {
        len = 1;
	c = 0;	/* Make the command line. */
	while( c < argc )
	{
	    len += strlen(argv[c]) + 1;	// the 1 byte plus is for the space. The last
					// '\0' is already counted when 'len = 1;'
	    cmdline = realloc(cmdline, len);
	
	    if(c) strcat(cmdline, argv[c]);
	    else  strcpy(cmdline, argv[c]);
	
	    strcat(cmdline, " ");
	    c++;
	}
        Client(report) = malloc(sizeof(Report)+len-1);	// The first byte of cmdline is
							// 'Report' structure.
	bzero(Client(report), sizeof(Report)+len-1);
	strcpy(Client(report)->cmd, cmdline);
	Client(report)->cmd_len = len;
	
	free(cmdline);
	cmdline = NULL;
    }

    return 0;	
}





/*
 * initialize data that were taken from the command line as well as the ones
 * depending on command line values.
 */
int initcmd(void)
{
    loff_t offt;

    if(!State.type){
	report(0,"You have to specify the program mode.\n\n");
	return -1;
    }
    
    /* Client mode */
    if(State.type == TP_CLI)
    {
	report(0, BLD"Client mode"RST".\n");
    
        if( !((Client(inputf)) ? !strcmp(Client(inputf), "-") : 1) )	/* if not input == stdin */
	    if( stat( Client(inputf), &Client(instat) ) < 0 ){
		report(0,"Cannot stat input: %s :: %s.\n", Client(inputf), strerror(errno));
		return -1;
	    }

        if( (Client(infd) = open(Client(inputf), O_RDONLY | O_LARGEFILE )) < 0 ){
	    report(0, "Failed opening input: %s :: %s.\n",
		( ((Client(inputf)) ? !strcmp(Client(inputf), "-") : 1) ) ?
		    "stdin" : Client(inputf),
		strerror(errno) );
	    return -1;
	}
	
	if( llseek(Client(infd), 0,0, &Client(byte_count), SEEK_END) < 0 ){
	    report(0,RED"Failed"RST": input file cannot be llseek()ed: %s\n", strerror(errno));
	    return -1;
	}
	llseek(Client(infd), 0,0, &offt, SEEK_SET);
	report(1,"Input size: %llu\n", Client(byte_count));
	
	if( Client(blocksize) < 1 ) Client(blocksize) = DEF_BLKSZ;
	if( Client(blocksize) > MAX_BLKSZ ) {
	    report(0, "Blocksize cannot exceed %u bytes.\n", MAX_BLKSZ);
	    return -1;
	}
	if( Client(port) < 1 ) Client(port) = DEF_PORT;

	if( !Client(ip) ){
	    report(0,"You must specify a host to connect to.\n");
	    return -1;
	}
	else {
	    struct hostent *h;
	    report(1,"Finding host: %s\n", Client(ip));
	    h = gethostbyname(Client(ip));
	    if( !h ){
		report(0,"Host not found.\n\n");
		return -1;
	    }
	    
	    report(2,"\tRemote host is: %s (%s)\n", h->h_name,
		inet_ntoa( *(struct in_addr *)h->h_addr_list[0] ));
	    free(Client(ip));
	    Client(ip) = strdup( inet_ntoa( *(struct in_addr *)h->h_addr_list[0] ) );
	}
    
	if( inet_addr(Client(ip)) == INADDR_NONE ){
	    report(0,"Invalid remote ip: %s.\n", Client(ip));
	    return -1;
	}
	
	if( Client(blocksize)%512 ){
	    report(0,"Invalid Block size! Must be multiple of 512\n");
	    return -1;
	}
    
	report(1,"Command line arguments parsed successfully.\n");
    
	report(2,"Report level: %d\n", State.reportlevel);
	report(2,"Remote Host: %s @ %u\n", Client(ip), Client(port));
	report(2,"Input location: %s (%d).\n", ( ((Client(inputf)) ? !strcmp(Client(inputf), "-") : 1) ) ?
	    "stdin" : Client(inputf), Client(infd));
	report(2,"Blocksize: %u.\n", Client(blocksize));
    }
    else if(State.type == TP_SRV)	/* Server mode */
    {
	report(0, BLD"Server mode"RST".\n");
	
	if( !Server(outputf) ){
	    report(0,"No output file specified.\n");
	    return -1;
	}
    
	if( (Server(spansz) > 0) && (Server(spansz) < MIN_SPANSZ) ){
	    report(0, "min span cannot be less than %u bytes.\n", MIN_SPANSZ);
	    return -1;
	}

	if( Server(spansz) ){
	    Server(outputf) = realloc( Server(outputf), strlen(Server(outputf)) + 2 + SPAN_FILE_DIGITS );
	    Server(span_base) = strdup(Server(outputf));
	    
		/* make a string for out file: "%s.%0XXd", where XX is SPAN_FILE_DIGITS */
	    sprintf( Server(span_fmt), "%%s.%%0%dd", SPAN_FILE_DIGITS );
	    
	    report(3,"Span enabled: files produced by: <%s>\n", Server(span_fmt));
	    sprintf( Server(outputf), Server(span_fmt), Server(span_base), 1 );
	    Server(span_cnt) = 1;
	}
	
	if( (Server(outfd) = open(Server(outputf), O_WRONLY | O_LARGEFILE | O_CREAT, 0666 )) < 0 ){
	    report(0, "Failed opening output: %s :: %s.\n", Server(outputf), strerror(errno) );
	    return -1;
	}
	
	if( !Server(logf) ) Server(logf) = DEF_LOG;
    
	if( Server(port) < 1 ) Server(port) = DEF_PORT;
    
	report(1,"Command line arguments parsed successfully.\n");
    
	report(2,"Report level: %d\n", State.reportlevel);
	report(2,"Listen port: %u\n", Server(port));
	if( !Server(spansz) )
	    report(2,"Output location: %s.\n", Server(outputf));
	else
	    report(2,"Output spanning enabled. Starting off: %s.\n", Server(outputf));
	report(2,"Log file will be: %s\n", Server(logf));
    }
    else {
	report(0, "Invalid mode.\n");
	return -1;
    }    
    
    return 0;
}







int main(int argc, char *argv[])
{
    struct timeval start, end;
    time_t st, nd;
    ulong u;

    if(argc<2){
	report(0,VERSION_HEADER"Try '%s -h' for help.\n",argv[0]);
	return 0;
    }

    bzero( &State, sizeof(State) );
	/*** Set some defaults here. ***/
//State.post.infd = State.post.outfd = State.post.sock = State.post.c_sock = -1;

    if( parsecmd(argc,argv) < 0 ){
	report(0,"Try '%s -h' for help.\n",argv[0]);
	goto fail;
    }
    
    if( initcmd() < 0 ){
	report(0,"Try '%s -h' for help.\n",argv[0]);
	goto fail;
    }

    if( initnet() < 0 ) goto fail;

    signal(SIGINT, sig_handler);
    signal(SIGPIPE, sig_handler);
    signal(SIGSEGV, sig_handler);
    
    if( State.type == TP_SRV ) if( waitclient() < 0 ) goto fail;

    printf("\033[?25l");	// hide the user's cursor.
    
    st = time(0);
    report(0,"Start: %s", ctime(&st));
    gettimeofday(&start, NULL);
    
    if( State.type == TP_CLI ){
	if( client_loop() < 0 ) goto fail;
    } else {
	if( server_loop() < 0 ) goto fail;
    }
    
    gettimeofday(&end, NULL);
    nd = time(0);
    report(0,"\nEnd:  %s", ctime(&nd));
    
    printf("\033[?25h");	// restore the user's cursor.
    
    u = end.tv_sec - start.tv_sec;
    if( !u ) u = 1;
    
    if( State.type == TP_CLI )
    {
	report(1,"All input data send. Now sending report.\n");
    
	if( initnet() < 0 ) goto fail;
    
	if( sendrest() < 0 ) goto fail;	/* We do error checking, to handle our
					 * exit status */
	closenet();
    
	report(0,"\n%s:\n%llu bytes in\n%llu bytes out\n",
	    argv[0], Client(tot_in), Client(tot_out));
    } else {
	report(1,"All data wrote. Now reading report.\n");
    
	if( waitclient() < 0 ) goto fail;
    
        if( readrest() < 0 ) goto fail;	/* We do error checking, to handle our
					 * exit status */
	closenet();

	savelog(st, nd, argc, argv, u, (float)(Server(tot_in) / 1024 / u));

        report(0,"\n%llu bytes read\n%llu bytes wrote\n%llu bytes reported by client.\n",
	    Server(tot_in), Server(tot_out), Server(remote_in));
    }
    
    report(0,BLD"%u"RST" seconds - "BLD"%.2f"RST" Kbytes / sec\n\n", u, (float)(Global(tot_in) / 1024 / u) );
    
    return 0;
fail:
    cleanup();
    return 1;
}
