/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * skiperr.c
 *
 * Read block (State.pre.blocksize sized) data from input when it's
 * known they contain an I/O error.
 */

#include <main.h>


    /* We use these to avoid HUGE code lines in the code arranging the
     * badblocks list, since we make a list where we have to make some
     * checks to see where each element should be inserted...
     */
#define Previous	Global(bbdefs)[ Global(report)->badblocks_no - 1 ]
#define Current		Global(bbdefs)[ Global(report)->badblocks_no ]
#define MakeCurrent	\
    {												\
	Global(bbdefs) = realloc(Global(bbdefs), sizeof(BBdef) * (Global(report)->badblocks_no+1));\
	if( !Global(bbdefs) ){									\
	    report(0,RED"Failed"RST" storing Badblock information: %s\n", strerror(errno));	\
	    cleanup();										\
	    exit(1);										\
	}											\
	bzero(&(Global(bbdefs)[Global(report)->badblocks_no]), sizeof(BBdef));			\
    }




/*
 * NOTE: We're ALWAYS been called by readinput_c().
 *
 * (The following are true due to the above statement...)
 *	-> State.tot_in contains the offset we are before the last read()
 *		that returned error.
 *	-> Buffer is initialized already (to 'State.pre.blocksize' bytes).
 *	-> The current file offset kept in the kernel structures is unknown,
 *		(man read).
 *	-> 'errno' has been set by read(). It HAS NOT CHANGED since then...
 *
 * report() should write on levels 0 (for important messages) and 1 (for
 * 	information). This is due to the fact that level 1 doesn't slow down
 *	the transfer loop (while no errors are encountered) and on the other
 *	hand within the context below, speed is by definition SLOW...
 */
 
 
#if ERR_RETRIES > 1    /* We keep two versions of skipp_error for optimization purposes according to */
		       /* our compile-time configuration. */


int skip_error(void)
{
    register uint i;
    register int len;
    register u_int64_t bb;
    loff_t offt;
    register ulong high, low;
    
    State.buflen = 0;
   
    report(0,"Retrying due to read errors...\n");

    while( State.buflen < Client(blocksize) )
    {
//	low = *(ulong*)&(Client(tot_in)+State.buflen);
//	high = *(ulong*)( ((char*)&(Client(tot_in)+State.buflen)) + sizeof(ulong) );
	bb = (Client(tot_in)+State.buflen);
	low =  (ulong) (bb & (u_int64_t)0x000000FFFFFF);
	high = (ulong)((bb & (u_int64_t)0xFFFFFF000000)>>32);
	
	if( llseek(Client(infd), high, low, &offt, SEEK_SET) < 0 )
	{
	    report(0,"llseek() error: %s\n", strerror(errno));
	    report(0,"Don't know what to do: Abort.\n");
	    return -1;
	}
	
	if( offt != (Client(tot_in)+State.buflen) )
	{
	    report(0,RED"Error (fatal)"RST": llseek returned correct seek but wrong offset is reported.\n");
	    report(0,"Abort.\n");
	    cleanup();
	    exit(1);
	}

	i = 0;
	while( i < ERR_RETRIES )
	{
	    i++;	// set this here for: proper report()s and for immediate continue;s.
	    len = read(Client(infd), State.buf+State.buflen, DEF_ERR_BLKSZ);
	    if( len < 0 ){
		report(1,RED"Error"RST" reading block (%lu). Try %u of %u.\n", DEF_ERR_BLKSZ, i, ERR_RETRIES);
		continue;
	    } else if(len == 0){
		report(0,BLU"EOF"RST" while skipping errors: Assuming normal condition...\n");
		Client(tot_in) += State.buflen;
		return State.buflen;
	    } else if(len < DEF_ERR_BLKSZ){	/* if not reading enough */
		char c[1];
		    /* Usually for testing, input file might be a regular file and NOT
		     * a disk or some other 512 bytes sector thing, so we might
		     * read less than 512
		     */
		if( read(Client(infd), c, 1) == 0 ){
		    report(0,BLU"EOF"RST" while skipping errors: Assuming normal condition...\n");
		    State.buflen += len;
		    Client(tot_in) += State.buflen;
		    return State.buflen;
		} else {
		    Client(report)->badcond = 1;
		    report(0, "Unexpected condition\n"
			      "Reading unreliable, Aborting.\n");
		    return -1;
		}
	    } else {
		report(1,GRN"Success"RST" reading block @ %llu!\n",
		    (unsigned long long)((Client(tot_in) + State.buflen) / 512) );
		break;
	    }
	}
	
	if( i == ERR_RETRIES )	/* Couldn't read the block, *&!@#^$)(@$#% */
	{
	    report(1,"Couldn't read block.\n");
	    bzero(State.buf+State.buflen, DEF_ERR_BLKSZ);

	    report(1,"Error reading block @ %llu. Retry failed.\n",
		(unsigned long long)((Client(tot_in) + State.buflen) / 512));
	    bzero(State.buf+State.buflen, DEF_ERR_BLKSZ);

	    if( !Client(bbdefs) )	/* The first one is ALWAYS written. */
	    {
		MakeCurrent;
		Current.bb_offt = (Client(tot_in) + State.buflen) / 512;
		report(4,"\t\tPut the first element (%llu).\n", Current.bb_offt);
		Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
	    }
	    else
	    {
		bb = (Client(tot_in) + State.buflen) / 512;
		
		if( Previous.bb_offt == (bb-1) )
		{
		    report(4,"\t\tSequence element.\n");
		    if( Previous.end ) Previous.bb_offt = bb;	/* Another element in a sequence: just update. */
		    else {
		        report(4,"\t\tSetting current element.\n");
			MakeCurrent;		/* We're the second element: behave as such.  */
			Current.bb_offt = bb;
			Current.end = 1;
			
			Previous.start = 1;

			report(4,"\t\tSet: %llu.\n", Current.bb_offt );
			Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
		    }
		} else {
		    report(4,"\t\tSetting a new element.\n");
		    MakeCurrent;		/* Nothing special: just add this element. */
		    Current.bb_offt = bb;
			
		    report(4,"\t\tSet: %llu.\n", Current.bb_offt );
		    Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
		}
	    }
	}
	State.buflen += DEF_ERR_BLKSZ;
    }

    report(1,"Finished with errors on block sized %u at offset %llu...\n\n",
	Client(blocksize), Client(tot_in));

    Client(tot_in) += State.buflen;
   
    return State.buflen;
}



#elif ERR_RETRIES == 1	 /* We keep two versions of skipp_error for optimization purposes according to */
		         /* our compile-time configuration. */


int skip_error(void)
{
    register int len;
    register u_int64_t bb;
    loff_t offt;
    register ulong high, low;
    
    State.buflen = 0;
   
    report(0,"Retrying due to read errors...\n");

    while( State.buflen < Client(blocksize) )
    {
//	low = *(ulong*)&(Client(tot_in)+State.buflen);
//	high = *(ulong*)( ((char*)&(Client(tot_in)+State.buflen)) + sizeof(ulong) );
	bb = (Client(tot_in)+State.buflen);
	low =  (ulong) (bb & (u_int64_t)0x000000FFFFFF);
	high = (ulong)((bb & (u_int64_t)0xFFFFFF000000)>>32);
	
	if( llseek(Client(infd), high, low, &offt, SEEK_SET) < 0 )
	{
	    report(0,"llseek() error: %s\n", strerror(errno));
	    report(0,"Don't know what to do: Abort.\n");
	    return -1;
	}

	len = read(Client(infd), State.buf+State.buflen, DEF_ERR_BLKSZ);
	if( len < 0 ){
	    report(1,RED"Error"RST" reading block @ %llu. Retry failed.\n",
		(unsigned long long)((Client(tot_in) + State.buflen) / 512));
	    bzero(State.buf+State.buflen, DEF_ERR_BLKSZ);

	    if( !Client(bbdefs) )	/* The first one is ALWAYS written. */
	    {
		MakeCurrent;
		Current.bb_offt = (Client(tot_in) + State.buflen) / 512;
		report(4,"\t\tPut the first element (%llu).\n", Current.bb_offt);
		Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
	    }
	    else
	    {
		bb = (Client(tot_in) + State.buflen) / 512;
		
		if( Previous.bb_offt == (bb-1) )
		{
		    report(4,"\t\tSequence element.\n");
		    if( Previous.end ) Previous.bb_offt = bb;	/* Another element in a sequence: just update. */
		    else {
		        report(4,"\t\tSetting current element.\n");
			MakeCurrent;		/* We're the second element: behave as such.  */
			Current.bb_offt = bb;
			Current.end = 1;
			
			Previous.start = 1;

			report(4,"\t\tSet: %llu.\n", Current.bb_offt );
			Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
		    }
		} else {
		    report(4,"\t\tSetting a new element.\n");
		    MakeCurrent;		/* Nothing special: just add this element. */
		    Current.bb_offt = bb;
			
		    report(4,"\t\tSet: %llu.\n", Current.bb_offt );
		    Client(report)->badblocks_no++;	/* Do this after any 'Previous' and 'Current' usage. */
		}
	    }
	} else if(len == 0){
	    report(0,BLU"EOF"RST" while skipping errors: Assuming normal condition...\n");
	    Client(tot_in) += State.buflen;
	    return State.buflen;
	} else if(len < DEF_ERR_BLKSZ){	/* if not reading enough */
	    char c[1];
	        /* Usually for testing, input file might be a regular file and NOT
	         * a disk or some other 512 bytes sector thing, so we might
	         * read less than 512
	         */
	    if( read(Client(infd), c, 1) == 0 ){
	        report(0,BLU"EOF"RST" while skipping errors: Assuming normal condition...\n");
	        State.buflen += len;
	        Client(tot_in) += State.buflen;
	        return State.buflen;
	    } else {
	        Client(report)->badcond = 1;
	        report(0, "Unexpected condition\n"
	    	      "Reading unreliable, Aborting.\n");
	        return -1;
	    }
	} else
	    report(1,GRN"Success"RST" reading block @ %llu!\n",
		(unsigned long long)((Client(tot_in) + State.buflen) / 512) );
	
	State.buflen += DEF_ERR_BLKSZ;
    }

    report(1,"Finished with errors on block sized %u at offset %llu...\n\n",
	Client(blocksize), Client(tot_in));

    Client(tot_in) += State.buflen;
   
    return State.buflen;
}


#else
#	error Bad ERR_RETRIES definion (should be >= 1)... Check its definition in main.h
#endif /* ERR_RETRIES */
