/*
 * Copyright 2002 MD5 S.A.
 * This is free software distributed under the GNU GPL 2. You
 * should have received a copy of the GPL 2 licence.
 * There is absolutely NO warranty for this software.
 */

/*
 * util.c
 *
 * Utilities.
 */

#include <main.h>


/*
 * Report something to the user if level is less than or equal to
 * the reportlevel set in our 'State'.
 */
void report(int level, char *fmt, ... )
{
    va_list l;		/* stdarg.h */
    if( level <= State.reportlevel ){
	va_start(l, fmt);
	vprintf(fmt, l);
	va_end(l);
    }
}



/*
 * print the help message.
 */
void printhelp(void)
{
    printf(VERSION_HEADER
"Usage::
General options:
  --send\tClient mode ( read data and send to network ).
  --recv\tServer mode ( read data from network and write locally ).
  -d\tReport level (default is 0).
  -p\t--port\n\t\tRemote port to connect to (default is %u).
  -m\t--checsum=[none|md5|crc32|both]
\t\tDefine the checksum algorythm(s). The default is md5.
  -s\t--silent\n\t\tPrint nothing on the screen.
  -h\t--help\tThis screen.
  -v\t--version\n\t\tPrint version info and exit.
 
Client options:
  -i\t--in\n\t\tInput location.
  -r\t--host\n\t\tRemote host ip.
  -b\t--bs\n\t\tRead blocks of size given (default is %d, max is %u).
  -c\t--count\n\t\tThe  number  of  blocks  of size BLOCK_SIZE to read
\t\tfrom the input location.
  -e\t--skip-errors\n\t\tDo not exit on read errors, instead replace bad blocks
\t\twith zero's. The server will receive a list with their locations.

Server options:
 -o\t--out\n\t\tOutput location.
 -a\t--span\n\t\tSpan size of output files (min is %u).
 -l\t--log\n\t\tLog file to write when finishes with all operations.
\t\tThe default is %s.
 -n\t--nolog\n\t\tTell the server not to write any log file.

", DEF_PORT, DEF_BLKSZ, MAX_BLKSZ, MIN_SPANSZ, DEF_LOG );
}





/*
 * client loop.
 */
int client_loop(void)
{
    register int i = 0;
    register int rd;
    
    if( Client(md5) ) md5_init_ctx( &Client(ctx) );
    if( Client(crc32) ) crc32_init( Client(crc_table), &Client(crc32sum) );
    
    Client(transf) = 1;

    for(;;)
    {
	if( (rd = readinput_c()) < 0 ) return -1;
	
	if( rd == 0 ) break;
	
	i++;
	if(!(i%LOOPS_REPORT))
	    report(0,"\033[1G %llu bytes in.", Client(tot_in));

	if( Client(md5) ) md5_process_bytes(State.buf, rd, &Client(ctx));
	if( Client(crc32) ) crc32_process(Client(crc_table), &Client(crc32sum), State.buf, rd);
	
	if( sendinput() < 0 ) return -1;
    }
    
    closeinput();
    closenet();

    if( Client(md5) ) md5_finish_ctx(&Client(ctx), Client(md5sum));
    if( Client(crc32) ) crc32_finalize( Client(crc_table), &Client(crc32sum) );
    
    Client(transf) = 0;

    return 0;
}





/*
 * server loop.
 */
int server_loop(void)
{
    register int i = 0;
    register int rd;
    
    if( Server(md5) ) md5_init_ctx(&Server(ctx));
    if( Server(crc32) ) crc32_init(Server(crc_table), &Server(crc32sum));

    for(;;)
    {
	if( (rd = readinput_s()) < 0 ) return -1;
	
	if( rd == 0 ) break;
	
	i++;
	if(!(i%LOOPS_REPORT)) report(0,"\033[1G %llu bytes in.", Server(tot_in));

	if( Server(md5) ) md5_process_bytes(State.buf, rd, &Server(ctx));
	if( Server(crc32) ) crc32_process(Server(crc_table), &Server(crc32sum), State.buf, rd);
	
	if( writeoutput() < 0 ) return -1;
    }
    
    if( Server(md5) ) md5_finish_ctx(&Server(ctx), Server(md5sum));
    if( Server(crc32) ) crc32_finalize(Server(crc_table), &Server(crc32sum));
    
    return 0;
}




/*
 * get the mac address of the given IP.
 */
char *macof(struct in_addr ip)
{
    FILE *fp;
    static char buf[256];
    char *ipstr = inet_ntoa(ip), *c;
    static char ret[18] = "";
    
    if( !(fp = fopen("/proc/net/arp", "r")) ){
	report(0,RED"Error"RST": Cannot open /proc/net/arp: %s\n", strerror(errno));
	return NULL;
    }
    
    while( fgets(buf, 256, fp) )
	if( strstr( buf, ipstr ) )
	{
	    c = buf;
	    while( (*c != '\0') && (*c != ':') ) c++;
	    
	    c -= 2;
	    strncpy( ret, c, 17 );
	    ret[17] = '\0';
	    break;
	}

    fclose(fp);
    
    return (ret[0]) ? ret : NULL;
}



/*
 * system call calling code.
 */
#undef llseek 	/* look in 'main.h' */
_syscall5(int, _llseek, uint, fd, ulong, high, ulong, low, loff_t *, offt, uint, wh);
